@@ -1,15 +1,16 @@
 use std::sync::PoisonError;
 use std::sync::RwLock;
 
+#[cfg(feature = "logs")]
+use crate::logs::LogError;
 #[cfg(feature = "metrics")]
 use crate::metrics::MetricsError;
+use crate::propagation::PropagationError;
 #[cfg(feature = "trace")]
 use crate::trace::TraceError;
+use once_cell::sync::Lazy;
 
-lazy_static::lazy_static! {
-    /// The global error handler.
-    static ref GLOBAL_ERROR_HANDLER: RwLock<Option<ErrorHandler>> = RwLock::new(None);
-}
+static GLOBAL_ERROR_HANDLER: Lazy<RwLock<Option<ErrorHandler>>> = Lazy::new(|| RwLock::new(None));
 
 /// Wrapper for error from both tracing and metrics part of open telemetry.
 #[derive(thiserror::Error, Debug)]
@@ -25,6 +26,17 @@ pub enum Error {
     #[error(transparent)]
     /// An issue raised by the metrics module.
     Metric(#[from] MetricsError),
+
+    #[cfg(feature = "logs")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "logs")))]
+    #[error(transparent)]
+    /// Failed to export logs.
+    Log(#[from] LogError),
+
+    #[error(transparent)]
+    /// Error happens when injecting and extracting information using propagators.
+    Propagation(#[from] PropagationError),
+
     #[error("{0}")]
     /// Other types of failures not covered by the variants above.
     Other(String),
@@ -47,11 +59,17 @@ pub fn handle_error<T: Into<Error>>(err: T) {
         _ => match err.into() {
             #[cfg(feature = "metrics")]
             #[cfg_attr(docsrs, doc(cfg(feature = "metrics")))]
-            Error::Metric(err) => eprintln!("OpenTelemetry metrics error occurred {:?}", err),
+            Error::Metric(err) => eprintln!("OpenTelemetry metrics error occurred. {}", err),
             #[cfg(feature = "trace")]
             #[cfg_attr(docsrs, doc(cfg(feature = "trace")))]
-            Error::Trace(err) => eprintln!("OpenTelemetry trace error occurred {:?}", err),
-            Error::Other(err_msg) => eprintln!("OpenTelemetry error occurred {}", err_msg),
+            Error::Trace(err) => eprintln!("OpenTelemetry trace error occurred. {}", err),
+            #[cfg(feature = "logs")]
+            #[cfg_attr(docsrs, doc(cfg(feature = "logs")))]
+            Error::Log(err) => eprintln!("OpenTelemetry log error occurred. {}", err),
+            Error::Propagation(err) => {
+                eprintln!("OpenTelemetry propagation error occurred. {}", err)
+            }
+            Error::Other(err_msg) => eprintln!("OpenTelemetry error occurred. {}", err_msg),
         },
     }
 }
