@@ -1,17 +1,14 @@
-//! # No-op OpenTelemetry Trace Implementation
+//! No-op trace impls
 //!
 //! This implementation is returned as the global tracer if no `Tracer`
 //! has been set. It is also useful for testing purposes as it is intended
 //! to have minimal resource utilization and runtime impact.
-use crate::trace::TraceResult;
 use crate::{
-    sdk::export::trace::{ExportResult, SpanData, SpanExporter},
-    trace,
-    trace::{SpanBuilder, TraceContextExt, TraceFlags, TraceState},
-    Context, KeyValue,
+    propagation::{text_map_propagator::FieldIter, Extractor, Injector, TextMapPropagator},
+    trace::{self, TraceContextExt as _},
+    Context, InstrumentationLibrary, KeyValue,
 };
-use async_trait::async_trait;
-use std::time::SystemTime;
+use std::{borrow::Cow, sync::Arc, time::SystemTime};
 
 /// A no-op instance of a `TracerProvider`.
 #[derive(Clone, Debug, Default)]
@@ -30,14 +27,9 @@ impl trace::TracerProvider for NoopTracerProvider {
     type Tracer = NoopTracer;
 
     /// Returns a new `NoopTracer` instance.
-    fn get_tracer(&self, _name: &'static str, _version: Option<&'static str>) -> Self::Tracer {
+    fn library_tracer(&self, _library: Arc<InstrumentationLibrary>) -> Self::Tracer {
         NoopTracer::new()
     }
-
-    /// Return an empty `Vec` as there isn't any span processors in `NoopTracerProvider`
-    fn force_flush(&self) -> Vec<TraceResult<()>> {
-        Vec::new()
-    }
 }
 
 /// A no-op instance of a `Span`.
@@ -46,40 +38,31 @@ pub struct NoopSpan {
     span_context: trace::SpanContext,
 }
 
-impl Default for NoopSpan {
-    fn default() -> Self {
-        NoopSpan::new()
-    }
-}
-
 impl NoopSpan {
-    /// Creates a new `NoopSpan` instance.
-    pub fn new() -> Self {
-        NoopSpan {
-            span_context: trace::SpanContext::new(
-                trace::TraceId::invalid(),
-                trace::SpanId::invalid(),
-                TraceFlags::default(),
-                false,
-                TraceState::default(),
-            ),
-        }
-    }
+    /// The default `NoopSpan`, as a constant
+    pub const DEFAULT: NoopSpan = NoopSpan {
+        span_context: trace::SpanContext::NONE,
+    };
 }
 
 impl trace::Span for NoopSpan {
     /// Ignores all events
-    fn add_event(&mut self, _name: String, _attributes: Vec<KeyValue>) {
-        // Ignore
+    fn add_event<T>(&mut self, _name: T, _attributes: Vec<KeyValue>)
+    where
+        T: Into<Cow<'static, str>>,
+    {
+        // Ignored
     }
 
     /// Ignores all events with timestamps
-    fn add_event_with_timestamp(
+    fn add_event_with_timestamp<T>(
         &mut self,
-        _name: String,
+        _name: T,
         _timestamp: SystemTime,
         _attributes: Vec<KeyValue>,
-    ) {
+    ) where
+        T: Into<Cow<'static, str>>,
+    {
         // Ignored
     }
 
@@ -99,12 +82,19 @@ impl trace::Span for NoopSpan {
     }
 
     /// Ignores status
-    fn set_status(&mut self, _code: trace::StatusCode, _message: String) {
+    fn set_status(&mut self, _status: trace::Status) {
         // Ignored
     }
 
     /// Ignores name updates
-    fn update_name(&mut self, _new_name: String) {
+    fn update_name<T>(&mut self, _new_name: T)
+    where
+        T: Into<Cow<'static, str>>,
+    {
+        // Ignored
+    }
+
+    fn add_link(&mut self, _span_context: trace::SpanContext, _attributes: Vec<KeyValue>) {
         // Ignored
     }
 
@@ -128,66 +118,45 @@ impl NoopTracer {
 }
 
 impl trace::Tracer for NoopTracer {
-    type Span = trace::NoopSpan;
-
-    /// Returns a `NoopSpan` as they are always invalid.
-    fn invalid(&self) -> Self::Span {
-        trace::NoopSpan::new()
-    }
-
-    /// Starts a new `NoopSpan` with a given context.
-    ///
-    /// If the context contains a valid span, it's span context is propagated.
-    fn start_with_context<T>(&self, name: T, cx: Context) -> Self::Span
-    where
-        T: Into<std::borrow::Cow<'static, str>>,
-    {
-        self.build(SpanBuilder::from_name_with_context(name, cx))
-    }
-
-    /// Starts a `SpanBuilder`.
-    fn span_builder<T>(&self, name: T) -> trace::SpanBuilder
-    where
-        T: Into<std::borrow::Cow<'static, str>>,
-    {
-        trace::SpanBuilder::from_name(name)
-    }
+    type Span = NoopSpan;
 
     /// Builds a `NoopSpan` from a `SpanBuilder`.
     ///
     /// If the span builder or the context's current span contains a valid span context, it is
     /// propagated.
-    fn build(&self, builder: trace::SpanBuilder) -> Self::Span {
-        let cx = builder.parent_context;
-        if cx.has_active_span() {
-            trace::NoopSpan {
-                span_context: cx.span().span_context().clone(),
-            }
-        } else {
-            self.invalid()
+    fn build_with_context(&self, _builder: trace::SpanBuilder, parent_cx: &Context) -> Self::Span {
+        NoopSpan {
+            span_context: parent_cx.span().span_context().clone(),
         }
     }
 }
 
-/// A no-op instance of an [`SpanExporter`].
+/// A no-op instance of an [`TextMapPropagator`].
 ///
-/// [`SpanExporter`]: crate::sdk::export::trace::SpanExporter
+/// [`TextMapPropagator`]: crate::propagation::TextMapPropagator
 #[derive(Debug, Default)]
-pub struct NoopSpanExporter {
+pub struct NoopTextMapPropagator {
     _private: (),
 }
 
-impl NoopSpanExporter {
-    /// Create a new noop span exporter
+impl NoopTextMapPropagator {
+    /// Create a new noop text map propagator
     pub fn new() -> Self {
-        NoopSpanExporter { _private: () }
+        NoopTextMapPropagator { _private: () }
     }
 }
 
-#[async_trait]
-impl SpanExporter for NoopSpanExporter {
-    async fn export(&mut self, _batch: Vec<SpanData>) -> ExportResult {
-        Ok(())
+impl TextMapPropagator for NoopTextMapPropagator {
+    fn inject_context(&self, _cx: &Context, _injector: &mut dyn Injector) {
+        // ignored
+    }
+
+    fn extract_with_context(&self, _cx: &Context, _extractor: &dyn Extractor) -> Context {
+        Context::current()
+    }
+
+    fn fields(&self) -> FieldIter<'_> {
+        FieldIter::new(&[])
     }
 }
 
@@ -195,7 +164,7 @@ impl SpanExporter for NoopSpanExporter {
 mod tests {
     use super::*;
     use crate::testing::trace::TestSpan;
-    use crate::trace::{self, Span, Tracer};
+    use crate::trace::{Span, TraceState, Tracer};
 
     fn valid_span_context() -> trace::SpanContext {
         trace::SpanContext::new(
@@ -210,17 +179,18 @@ mod tests {
     #[test]
     fn noop_tracer_defaults_to_invalid_span() {
         let tracer = NoopTracer::new();
-        let span = tracer.start_with_context("foo", Context::new());
+        let span = tracer.start_with_context("foo", &Context::new());
         assert!(!span.span_context().is_valid());
     }
 
     #[test]
     fn noop_tracer_propagates_valid_span_context_from_builder() {
         let tracer = NoopTracer::new();
-        let builder = tracer
-            .span_builder("foo")
-            .with_parent_context(Context::new().with_span(TestSpan(valid_span_context())));
-        let span = tracer.build(builder);
+        let builder = tracer.span_builder("foo");
+        let span = tracer.build_with_context(
+            builder,
+            &Context::new().with_span(TestSpan(valid_span_context())),
+        );
         assert!(span.span_context().is_valid());
     }
 
@@ -230,7 +200,7 @@ mod tests {
         let cx = Context::new().with_span(NoopSpan {
             span_context: valid_span_context(),
         });
-        let span = tracer.start_with_context("foo", cx);
+        let span = tracer.start_with_context("foo", &cx);
         assert!(span.span_context().is_valid());
     }
 
@@ -238,7 +208,7 @@ mod tests {
     fn noop_tracer_propagates_valid_span_context_from_remote_span_context() {
         let tracer = NoopTracer::new();
         let cx = Context::new().with_remote_span_context(valid_span_context());
-        let span = tracer.start_with_context("foo", cx);
+        let span = tracer.start_with_context("foo", &cx);
         assert!(span.span_context().is_valid());
     }
 }
